/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_TRPCLONG4CAN_H
#define IMPL_TYPE_TRPCLONG4CAN_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint8.h"

struct TRPCLong4CAN {
    ::Float Long_OP_Vx_des_tar{ 0.0F };
    ::Float Long_OP_Ax_tar{ 0.0F };
    ::UInt8 Long_OP_AccVlcMainReq{ 0U };
    ::UInt8 Long_OP_Dec2Stop_Out{ 0U };
    ::UInt8 Long_OP_DriveoffReq_out{ 0U };
    ::UInt8 Long_OP_VlcECEnable{ 0U };
    ::Float Long_OP_MProp{ 0.0F };
    ::Float Long_OP_VlcAxTar{ 0.0F };
    ::UInt8 Long_OP_VlcDCEnable{ 0U };
    ::Float Long_OP_VLC_a_TRP2VLC_Ax_Tar{ 0.0F };
    ::UInt8 Long_OP_Vlc_AxTar_Flg{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(Long_OP_Vx_des_tar);
        fun(Long_OP_Ax_tar);
        fun(Long_OP_AccVlcMainReq);
        fun(Long_OP_Dec2Stop_Out);
        fun(Long_OP_DriveoffReq_out);
        fun(Long_OP_VlcECEnable);
        fun(Long_OP_MProp);
        fun(Long_OP_VlcAxTar);
        fun(Long_OP_VlcDCEnable);
        fun(Long_OP_VLC_a_TRP2VLC_Ax_Tar);
        fun(Long_OP_Vlc_AxTar_Flg);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(Long_OP_Vx_des_tar);
        fun(Long_OP_Ax_tar);
        fun(Long_OP_AccVlcMainReq);
        fun(Long_OP_Dec2Stop_Out);
        fun(Long_OP_DriveoffReq_out);
        fun(Long_OP_VlcECEnable);
        fun(Long_OP_MProp);
        fun(Long_OP_VlcAxTar);
        fun(Long_OP_VlcDCEnable);
        fun(Long_OP_VLC_a_TRP2VLC_Ax_Tar);
        fun(Long_OP_Vlc_AxTar_Flg);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("Long_OP_Vx_des_tar", Long_OP_Vx_des_tar);
        fun("Long_OP_Ax_tar", Long_OP_Ax_tar);
        fun("Long_OP_AccVlcMainReq", Long_OP_AccVlcMainReq);
        fun("Long_OP_Dec2Stop_Out", Long_OP_Dec2Stop_Out);
        fun("Long_OP_DriveoffReq_out", Long_OP_DriveoffReq_out);
        fun("Long_OP_VlcECEnable", Long_OP_VlcECEnable);
        fun("Long_OP_MProp", Long_OP_MProp);
        fun("Long_OP_VlcAxTar", Long_OP_VlcAxTar);
        fun("Long_OP_VlcDCEnable", Long_OP_VlcDCEnable);
        fun("Long_OP_VLC_a_TRP2VLC_Ax_Tar", Long_OP_VLC_a_TRP2VLC_Ax_Tar);
        fun("Long_OP_Vlc_AxTar_Flg", Long_OP_Vlc_AxTar_Flg);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("Long_OP_Vx_des_tar", Long_OP_Vx_des_tar);
        fun("Long_OP_Ax_tar", Long_OP_Ax_tar);
        fun("Long_OP_AccVlcMainReq", Long_OP_AccVlcMainReq);
        fun("Long_OP_Dec2Stop_Out", Long_OP_Dec2Stop_Out);
        fun("Long_OP_DriveoffReq_out", Long_OP_DriveoffReq_out);
        fun("Long_OP_VlcECEnable", Long_OP_VlcECEnable);
        fun("Long_OP_MProp", Long_OP_MProp);
        fun("Long_OP_VlcAxTar", Long_OP_VlcAxTar);
        fun("Long_OP_VlcDCEnable", Long_OP_VlcDCEnable);
        fun("Long_OP_VLC_a_TRP2VLC_Ax_Tar", Long_OP_VLC_a_TRP2VLC_Ax_Tar);
        fun("Long_OP_Vlc_AxTar_Flg", Long_OP_Vlc_AxTar_Flg);
    }

    friend bool operator==(const ::TRPCLong4CAN& lhs, const ::TRPCLong4CAN& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.Long_OP_Vx_des_tar - rhs.Long_OP_Vx_des_tar)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.Long_OP_Ax_tar - rhs.Long_OP_Ax_tar)) < DBL_EPSILON) && (lhs.Long_OP_AccVlcMainReq == rhs.Long_OP_AccVlcMainReq) && (lhs.Long_OP_Dec2Stop_Out == rhs.Long_OP_Dec2Stop_Out) && (lhs.Long_OP_DriveoffReq_out == rhs.Long_OP_DriveoffReq_out) && (lhs.Long_OP_VlcECEnable == rhs.Long_OP_VlcECEnable) && (fabs(static_cast<double>(lhs.Long_OP_MProp - rhs.Long_OP_MProp)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.Long_OP_VlcAxTar - rhs.Long_OP_VlcAxTar)) < DBL_EPSILON) && (lhs.Long_OP_VlcDCEnable == rhs.Long_OP_VlcDCEnable) && (fabs(static_cast<double>(lhs.Long_OP_VLC_a_TRP2VLC_Ax_Tar - rhs.Long_OP_VLC_a_TRP2VLC_Ax_Tar)) < DBL_EPSILON) && (lhs.Long_OP_Vlc_AxTar_Flg == rhs.Long_OP_Vlc_AxTar_Flg);
    }
};


#endif // IMPL_TYPE_TRPCLONG4CAN_H
